@@ -17,10 +17,11 @@
 #    NOTE: filesystems that span multiple underlying filesystems/devices (eg lvm VG) should specify those in params, separated by colons.  \
 #          the <blockdevice> in the beginning doesn't matter much, it can be pretty much any device, or not existent, i think.  But it's probably best to make it one of the devices listed in params
 #          no '+' characters allowed for devices in $fs_params (eg use the real names)
-
+#    NOTE: if you want spaces in params or opts, you must "encode" them by replacing them with 2 underscores.
 
 # -- ADDITIONAL INTERNAL FORMATS --
 # $TMP_FILESYSTEMS: each filesystem on a separate line, so block devices can appear multiple times be on multiple lines (eg LVM volumegroups with more lvm LV's)
+# uses spaces for separation (so you can easily do 'while.. read' loops. so fs_params and fs_opts are encoded here also
 # part part_type part_label fs_type fs_create fs_mountpoint fs_mount fs_opts fs_label fs_params
 
 
@@ -65,12 +66,12 @@ target_special_fs ()
 # Disable swap and umount all mounted filesystems for the target system in the correct order. (eg first $var_TARGET_DIR/a/b/c, then $var_TARGET_DIR/a/b, then $var_TARGET_DIR/a until lastly $var_TARGET_DIR
 target_umountall()
 {
-	infofy "Disabling all swapspace..." disks
+	inform "Disabling all swapspace..." disks
 	swapoff -a >/dev/null 2>&1
 	declare target=${var_TARGET_DIR//\//\\/} # escape all slashes otherwise awk complains
 	for mountpoint in $(mount | awk "/\/$target/ {print \$3}" | sort | tac )
 	do
-		infofy "Unmounting mountpoint $mountpoint" disks
+		inform "Unmounting mountpoint $mountpoint" disks
 		umount $mountpoint >/dev/null 2>$LOG
 	done
 }
@@ -198,11 +199,14 @@ getlabel()
 # $1 set to 1 to echo a newline after partition instead of a space (optional)
 # $2 extra things to echo for each partition (optional)
 # $3 something to append directly after the partition (optional) TODO: refactor code so there's a space in between, merge $2 and $3. use echo -e to print whatever user wants
-findpartitions() {
+findblockdevices() {
 	workdir="$PWD"
 	for devpath in $(finddisks)
 	do
 		disk=$(echo $devpath | sed 's|.*/||')
+		echo -n "/dev/$disk$3"
+		[ "$1" = 1 ] && echo || echo -n ' '
+		[ "$2" ] && echo $2
 		cd /sys/block/$disk   
 		for part in $disk*
 		do
@@ -265,7 +269,7 @@ findpartitions() {
 # taken from setup
 get_grub_map() {
 	rm $TMP_DEV_MAP &>/dev/null #TODO: this doesn't exist? is this a problem? ASKDEV
-	infofy "Generating GRUB device map...\nThis could take a while.\n\n Please be patient."
+	inform "Generating GRUB device map...\nThis could take a while.\n\n Please be patient."
 	$var_TARGET_DIR/sbin/grub --no-floppy --device-map $TMP_DEV_MAP >/tmp/grub.log 2>&1 <<EOF
 quit
 EOF
@@ -402,11 +406,12 @@ process_disks ()
 
 process_disk ()
 {
-	infofy "Partitioning $1" disks
+	inform "Partitioning $1" disks
 	partition $1 "$2"
 }
 
 # $1 fs_string
+# $2 decode yes/no
 parse_filesystem_string ()
 {
 	fs="$1"
@@ -417,6 +422,16 @@ parse_filesystem_string ()
 	fs_opts=`       cut -d ';' -f 5 <<< $fs`
 	fs_label=`      cut -d ';' -f 6 <<< $fs`
 	fs_params=`     cut -d ';' -f 7 <<< $fs`
+	if [ "$2" == 'yes' ]; then
+		fs_opts="${fs_opts//__/ }"
+		fs_params="${fs_params//__/ }"
+		[ "$fs_type"       = no_type       ] && fs_type=
+		[ "$fs_mountpoint" = no_mountpoint ] && fs_mountpoint=
+		[ "$fs_mount"      = no_mount      ] && fs_mount=
+		[ "$fs_opts"       = no_opts       ] && fs_opts=
+		[ "$fs_label"      = no_label      ] && fs_label=
+		[ "$fs_params"     = no_params     ] && fs_params=
+	fi
 }
 
 
@@ -449,7 +464,7 @@ process_filesystems ()
 	# phase 1: create all blockdevices and filesystems in the correct order (for each fs, the underlying block/lvm/devicemapper device must be available so dependencies must be resolved. for lvm:first pv's, then vg's, then lv's etc)
 	# don't let them mount yet. we take care of all that ourselves in the next phase
 
-	infofy "Phase 1: Creating filesystems & blockdevices" disks
+	inform "Phase 1: Creating filesystems & blockdevices" disks
 	done_filesystems=
 	for i in `seq 1 10`
 	do
@@ -467,17 +482,17 @@ process_filesystems ()
 					if [ "$part_type" = lvm-vg ] && vgdisplay $part | grep -q 'VG Name' # $part is a lvm VG and it exists. note that vgdisplay exists 0 when the requested vg doesn't exist.
 					then
 						debug 'FS' "$fs_id ->Still need to do it: Making the filesystem on a vg volume"
-						infofy "Making $fs_type filesystem on $part" disks
+						inform "Making $fs_type filesystem on $part" disks
 						process_filesystem $part $fs_type $fs_create $fs_mountpoint no_mount $fs_opts $fs_label $fs_params && done_filesystems+=("$fs_id") || returncode=1
 					elif [ "$part_type" != lvm-pv -a -b "$part" ] # $part is not a lvm PV and it exists
 					then
 						debug 'FS' "$fs_id ->Still need to do it: Making the filesystem on a non-pv volume"
-						infofy "Making $fs_type filesystem on $part" disks
+						inform "Making $fs_type filesystem on $part" disks
 						process_filesystem $part $fs_type $fs_create $fs_mountpoint no_mount $fs_opts $fs_label $fs_params && done_filesystems+=("$fs_id") || returncode=1
-					elif [ "$part_type" = lvm-pv ] && pvdisplay ${fs_params//:/ } >/dev/null # $part is a lvm PV. all needed lvm pv's exist. note that pvdisplay exits 5 as long as one of the args doesn't exist
+					elif [ "$part_type" = lvm-pv ] && pvdisplay ${fs_params//__/ } >/dev/null # $part is a lvm PV. all needed lvm pv's exist. note that pvdisplay exits 5 as long as one of the args doesn't exist
 					then
 						debug 'FS' "$fs_id ->Still need to do it: Making the filesystem on a pv volume"
-						infofy "Making $fs_type filesystem on $part" disks
+						inform "Making $fs_type filesystem on $part" disks
 						process_filesystem ${part/+/} $fs_type $fs_create $fs_mountpoint no_mount $fs_opts $fs_label $fs_params && done_filesystems+=("$fs_id") || returncode=1
 					else
 						debug 'FS' "$fs_id ->Cannot do right now..."
@@ -494,22 +509,22 @@ process_filesystems ()
 
 	# phase 2: mount all filesystems in the vfs in the correct order. (also swapon where appropriate)
 
-	infofy "Phase 2: Mounting filesystems" disks
+	inform "Phase 2: Mounting filesystems" disks
 	while read part part_type part_label fs_type fs_create fs_mountpoint fs_mount fs_opts fs_label fs_params
 	do
 		if [ "$fs_mountpoint" != no_mountpoint ]
 		then
-			infofy "Mounting $part ($fs_type) on $fs_mountpoint" disks
+			inform "Mounting $part ($fs_type) on $fs_mountpoint" disks
 			process_filesystem $part $fs_type no $fs_mountpoint $fs_mount $fs_opts $fs_label $fs_params || returncode=1
 		elif [ "$fs_type" = swap ]
 		then
-			infofy "Swaponning $part" disks
+			inform "Swaponning $part" disks
 			process_filesystem $part $fs_type no $fs_mountpoint $fs_mount $fs_opts $fs_label $fs_params || returncode=1
 		fi
 	done < <(sort -t \  -k 6 $TMP_FILESYSTEMS)
 
 	BLOCK_ROLLBACK_USELESS=0
-	[ $returncode -eq 0 ] && infofy "Done processing filesystems/blockdevices" disks 1 && return 0
+	[ $returncode -eq 0 ] && inform "Done processing filesystems/blockdevices" disks 1 && return 0
 	return $returncode
 }
 
@@ -517,7 +532,7 @@ process_filesystems ()
 # Roll back all "filesystems" (normal ones and dm-mapper based stuff) specified in $BLOCK_DATA.  Not partitions.  Doesn't restore data after you erased it, of course.
 rollback_filesystems ()
 {
-	infofy "Rolling back filesystems..." disks
+	inform "Rolling back filesystems..." disks
 	generate_filesystem_list
 	local warnings=
 	rm -f $TMP_FSTAB
@@ -525,20 +540,20 @@ rollback_filesystems ()
 	# phase 1: destruct all mounts in the vfs and swapoff swap volumes who are listed in $BLOCK_DATA
 	# re-order list so that we umount in the correct order. eg first umount /a/b/c, then /a/b. we sort alphabetically, which has the side-effect of sorting by stringlength, hence by vfs dependencies.
 
-	infofy "Phase 1: Umounting all specified mountpoints" disks
+	inform "Phase 1: Umounting all specified mountpoints" disks
 	done_umounts= # We translate some devices back to their original (eg /dev/sda3+ -> /dev/sda3 for lvm PV's). No need to bother user twice for such devices.
 	while read part part_type part_label fs_type fs_create fs_mountpoint fs_mount fs_opts fs_label fs_params
 	do
 		if [ "$fs_type" = swap ]
 		then
-			infofy "(Maybe) Swapoffing $part" disks
+			inform "(Maybe) Swapoffing $part" disks
 			swapoff $part &>/dev/null # could be that it was not swappedon yet.  that's not a problem at all.
 		elif [ "$fs_mountpoint" != no_mountpoint ]
 		then
 			part_real=${part/+/}
 			if ! check_is_in "$part_real" "${done_umounts[@]}"
 			then
-				infofy "(Maybe) Umounting $part_real" disks
+				inform "(Maybe) Umounting $part_real" disks
 				if mount | grep -q "^$part_real " # could be that this was not mounted yet. no problem, we can just skip it then.
 				then
 					if umount $part_real >$LOG
@@ -563,7 +578,7 @@ rollback_filesystems ()
 	#                      -> easier (implemented)
 
 
-	infofy "Phase 2: destructing relevant blockdevices" disks
+	inform "Phase 2: destructing relevant blockdevices" disks
 	for i in `seq 1 10`
 	do
 		open_items=0
@@ -577,7 +592,7 @@ rollback_filesystems ()
 			# there is probably a nice regex to check this but i'm bad at regexes.
 			if echo "$fs_string" | grep -q ';yes;/' || echo "$fs_string" | grep -q ';yes;no_mountpoint'
 			then
-				infofy "Skipping destruction of $part ($part_type) because one of the filesystems on it contains data you want to keep"
+				inform "Skipping destruction of $part ($part_type) because one of the filesystems on it contains data you want to keep"
 				# TODO: it's possible that if we don't clear a blockdevice here because there is something on it with valuable data, that this blockdevice itself is hosted on some other blockdevice (eg lvm VG,PV or dm_crypt), \
 				#       that blockdevice cannot be cleared as well because it depends on this one, so after 10 iterations the user will get a warning that not everything is cleared.  so we should fix this someday.
 			elif [ "$part_type" = dm_crypt ] # Can be in use for: lvm-pv or raw. we don't need to care about raw (it will be unmounted so it can be destroyed)
@@ -589,7 +604,7 @@ rollback_filesystems ()
 						debug 'FS' "$part ->Cannot do right now..."
 						open_items=1
 					else
-						infofy "Attempting destruction of device $part (type $part_type)" disks
+						inform "Attempting destruction of device $part (type $part_type)" disks
 						if ! cryptsetup luksClose $real_part &>$LOG
 						then
 							warnings="$warnings\nCould not cryptsetup luksClose $real_part"
@@ -608,7 +623,7 @@ rollback_filesystems ()
 						debug 'FS' "$part ->Cannot do right now..."
 						open_items=1
 					else
-						infofy "Attempting destruction of device $part (type $part_type)" disks
+						inform "Attempting destruction of device $part (type $part_type)" disks
 						if ! pvremove $real_part &>$LOG
 						then
 							warnings="$warnings\nCould not pvremove $part"
@@ -628,7 +643,7 @@ rollback_filesystems ()
 						debug 'FS' "$part ->Cannot do right now..."
 						open_items=1
 					else
-						infofy "Attempting destruction of device $part (type $part_type)" disks
+						inform "Attempting destruction of device $part (type $part_type)" disks
 						if ! vgremove $part &>$LOG # we shouldn't need -f because we clean up the lv's first.
 						then
 							warnings="$warnings\nCould not vgremove $part"
@@ -647,7 +662,7 @@ rollback_filesystems ()
 						debug 'FS' "$part ->Cannot do right now..."
 						open_items=1
 					else
-						infofy "Attempting destruction of device $part (type $part_type)" disks
+						inform "Attempting destruction of device $part (type $part_type)" disks
 						if ! lvremove -f $part &>$LOG
 						then
 							warnings="$warnings\nCould not lvremove -f $part"
@@ -669,8 +684,8 @@ rollback_filesystems ()
 		warnings="$warnings\nCould not destruct all filesystems/blockdevices.  It appears some depending filesystems/blockdevices could not be cleared in 10 iterations"
 		show_warning "Filesystem/blockdevice processor problem" "Warning: Could not destruct all filesystems/blockdevices.  It appears some depending filesystems/blockdevices could not be cleared in 10 iterations"
 	fi
-	[ -n "$warnings" ] && infofy "Rollback failed" disks 1 && show_warning "Rollback problems" "Some problems occurred while rolling back: $warnings.\n Thisk needs to be fixed before retrying disk/filesystem creation or restarting the installer" && return 1
-	infofy "Rollback succeeded" disks 1
+	[ -n "$warnings" ] && inform "Rollback failed" disks 1 && show_warning "Rollback problems" "Some problems occurred while rolling back: $warnings.\n Thisk needs to be fixed before retrying disk/filesystem creation or restarting the installer" && return 1
+	inform "Rollback succeeded" disks 1
 	done_filesystems=
 	BLOCK_ROLLBACK_USELESS=1
 	return 0
@@ -678,6 +693,7 @@ rollback_filesystems ()
 
 
 # make a filesystem on a blockdevice and mount if needed.
+# parameters: (all encoded)
 # $1 partition
 # $2 fs_type
 # $3 fs_create     (optional. defaults to yes)
@@ -693,20 +709,8 @@ process_filesystem ()
 	[ -z "$2" ]              && die_error "process_filesystem needs a filesystem type as \$2"
 	debug 'FS' "process_filesystem $@"
 	local ret=0
-
-        part=$1
-        fs_type=$2
-	fs_create=${3:-yes}
-	fs_mountpoint=${4:-no_mountpoint}
-	fs_mount=${5:-no_mount}
-	fs_opts=${6:-no_opts}
-	fs_label=${7:-no_label}
-	fs_params=${8:-no_params}
-	[ "$fs_mountpoint" = no_mountpoint ] && fs_mountpoint=
-	[ "$fs_mount"      = no_mount      ] && fs_mount=
-	[ "$fs_opts"       = no_opts       ] && fs_opts=
-	[ "$fs_label"      = no_label      ] && fs_label=
-	[ "$fs_params"     = no_params     ] && fs_params=
+	part=$1
+	parse_filesystem_string "$2;$3;$4;$5;$6;$7;$8" yes
 
 	# Create the FS
 	if [ "$fs_create" = yes ]
@@ -719,33 +723,36 @@ process_filesystem ()
 		[ -z "$fs_label" ] && [ "$fs_type" = lvm-vg -o "$fs_type" = lvm-pv ] && fs_label=default #TODO. implement the incrementing numbers label for lvm vg's and lv's
 
 		#TODO: health checks on $fs_params etc
-		case ${fs_type} in #TODO implement options (opts, fs_opts, whatever)
-			xfs)      mkfs.xfs -f $part           $opts >$LOG 2>&1; ret=$? ;;
-			jfs)      yes | mkfs.jfs $part        $opts >$LOG 2>&1; ret=$? ;;
-			reiserfs) yes | mkreiserfs $part      $opts >$LOG 2>&1; ret=$? ;;
-			ext2)     mke2fs "$part"              $opts >$LOG 2>&1; ret=$? ;;
-			ext3)     mke2fs -j $part             $opts >$LOG 2>&1; ret=$? ;;
-			ext4)     mkfs.ext4 $part             $opts >$LOG 2>&1; ret=$? ;; #TODO: installer.git uses mke2fs -t ext4 -O dir_index,extent,uninit_bg , which is best?
-			vfat)     mkfs.vfat $part             $opts >$LOG 2>&1; ret=$? ;;
+		case ${fs_type} in 
+			xfs)      mkfs.xfs -f $part           $fs_opts >$LOG 2>&1; ret=$? ;;
+			jfs)      yes | mkfs.jfs $part        $fs_opts >$LOG 2>&1; ret=$? ;;
+			reiserfs) yes | mkreiserfs $part      $fs_opts >$LOG 2>&1; ret=$? ;;
+			ext2)     mke2fs "$part"              $fs_opts >$LOG 2>&1; ret=$? ;;
+			ext3)     mke2fs -j $part             $fs_opts >$LOG 2>&1; ret=$? ;;
+			ext4)     mkfs.ext4 $part             $fs_opts >$LOG 2>&1; ret=$? ;; #TODO: installer.git uses mke2fs -t ext4 -O dir_index,extent,uninit_bg , which is best?
+			vfat)     mkfs.vfat $part             $fs_opts >$LOG 2>&1; ret=$? ;;
 			swap)     if [ -z "$fs_label" ]; then
-				  	mkswap $part          $opts >$LOG 2>&1; ret=$?
+				  	mkswap $part          $fs_opts >$LOG 2>&1; ret=$?
 				  else
-				  	mkswap -L $fs_label $part $opts >$LOG 2>&1; ret=$?
+				  	mkswap -L $fs_label $part $fs_opts >$LOG 2>&1; ret=$?
 				  fi
 				  ;;
 			dm_crypt) [ -z "$fs_params" ] && fs_params='-c aes-xts-plain -y -s 512';
-			          fs_params=${fs_params//_/ }
-                      infofy "Please enter your passphrase to encrypt the device (with confirmation)"
-			          cryptsetup $fs_params $opts luksFormat -q $part >$LOG 2>&1 < /dev/tty ; ret=$? #hack to give cryptsetup the approriate stdin. keep in mind we're in a loop (see process_filesystems where something else is on stdin)
-                      infofy "Please enter your passphrase to unlock the device"
+                      inform "Please enter your passphrase to encrypt the device (with confirmation)"
+			          cryptsetup $fs_params $fs_opts luksFormat -q $part >$LOG 2>&1 < /dev/tty ; ret=$? #hack to give cryptsetup the approriate stdin. keep in mind we're in a loop (see process_filesystems where something else is on stdin)
+                      inform "Please enter your passphrase to unlock the device"
 			          cryptsetup       luksOpen $part $fs_label >$LOG 2>&1 < /dev/tty; ret=$? || ( show_warning 'cryptsetup' "Error luksOpening $part on /dev/mapper/$fs_label" ) ;;
 			lvm-pv)   pvcreate $fs_opts $part              >$LOG 2>&1; ret=$? ;;
-			lvm-vg)   # $fs_params: ':'-separated list of PV's
-			          vgcreate $fs_opts $fs_label ${fs_params//:/ }      >$LOG 2>&1; ret=$? ;;
+			lvm-vg)   # $fs_params: list of PV's
+			          vgcreate $fs_opts $fs_label $fs_params      >$LOG 2>&1; ret=$? ;;
 			lvm-lv)   # $fs_params = size string (eg '5G')
-			          lvcreate -L $fs_params $fs_opts -n $fs_label `sed 's#/dev/mapper/##' <<< $part`   >$LOG 2>&1; ret=$? ;; #$opts is usually something like -L 10G # Strip '/dev/mapper/' part because device file may not exist.  TODO: do i need to activate them?
+			          lvcreate -L $fs_params $fs_opts -n $fs_label `sed 's#/dev/mapper/##' <<< $part`   >$LOG 2>&1; ret=$? ;; #$fs_opts is usually something like -L 10G # Strip '/dev/mapper/' part because device file may not exist.  TODO: do i need to activate them?
 			# don't handle anything else here, we will error later
 		esac
+		# The udevadm settle is a workaround for a bug/racecondition in cryptsetup. See:
+		# http://mailman.archlinux.org/pipermail/arch-releng/2010-April/000974.html
+		# It may not be needed (anymore), or not after every type of FS, but we're rather safe then sorry. and it seems to always return in less then a second
+		udevadm settle
 		BLOCK_ROLLBACK_USELESS=0
 		[ "$ret" -gt 0 ] && { show_warning "process_filesystem error" "Error creating filesystem $fs_type on $part."; return 1; }
 		sleep 2
