@@ -121,7 +121,7 @@ interactive_configure_system()
 interactive_timezone () {
 	ask_timezone || return 1
         TIMEZONE=$ANSWER_TIMEZONE
-        infofy "Setting Timezone to $TIMEZONE"
+        inform "Setting Timezone to $TIMEZONE"
 		if [ -n "$TIMEZONE" -a -e "/usr/share/zoneinfo/$TIMEZONE" ]
 		then
 			# This changes probably also the systemtime (UTC->$TIMEZONE)!
@@ -154,7 +154,7 @@ interactive_time () {
                 "return" "Looks good. back to main menu" "${EXTRA[@]}" "manual" "Set time and date manually" || return 1
                 if [ "$ANSWER_OPTION" = ntp ]
                 then
-			infofy "Syncing clock with internet pool ..."
+			inform "Syncing clock with internet pool ..."
 			if ntpdate pool.ntp.org >/dev/null
 			then
 				notify "Synced clock with internet pool successfully."
@@ -185,12 +185,6 @@ interactive_prepare_disks ()
 	NEXTITEM=
 	DISK_CONFIG_TYPE=
 	[ "$BLOCK_ROLLBACK_USELESS" = "0" ] && show_warning "Rollback may be needed" "It seems you already went here.  You should probably rollback previous changes before reformatting, otherwise stuff will probably fail"
-	[ -z "$PART_ACCESS" ] && PART_ACCESS=dev
-	ask_option $PART_ACCESS 'Partition Access Method' 'How do you want your partitions to be accessed in grubs menu.lst and /etc/fstab?' '' \
-		"dev"	"directly by /dev/* (most intuitive but devicefile names can change on kernel updates)" \
-		"label"	"by Disk-Label (Will use the filesystem labels where you give them, and fall back on 'dev' otherwise)" \
-		"uuid"	"by Universally Unique Identifier (You don't need to do anything, but doesn't look pretty)" || return 1
-	PART_ACCESS=$ANSWER_OPTION
 	while [ "$DONE" = "0" ]
 	do
 		rollbackstr=" (you don't need to do this)"
@@ -228,7 +222,7 @@ interactive_prepare_disks ()
 				then
 					if rollback_filesystems #TODO: this part doesn't belong here. move it to ui-interactive. (interactive_rollback)
 					then
-						infofy "Rollback succeeded"
+						inform "Rollback succeeded"
 					else
 						show_warning "Rollback failed" "Rollback failed"
 					fi
@@ -363,6 +357,7 @@ interactive_partition() {
 # but I think it's better to go through them all and by default always show the previous choice.
 interactive_filesystem ()
 {
+	# note: fs_mount: we dont need to ask this to the user. this is always 'target' for 99.99% of the users
 	local part=$1       # must be given and (scheduled to become) a valid device -> don't do [ -b "$1" ] because the device might not exist *yet*
 	local part_type=$2  # a part should always have a type
 	local part_label=$3 # can be empty
@@ -377,26 +372,14 @@ interactive_filesystem ()
 	NEW_FILESYSTEM=
 	if [ -n "$fs_string" ]
 	then
-		fs_type=`       cut -d ';' -f 1 <<< $fs_string`
-		fs_create=`     cut -d ';' -f 2 <<< $fs_string` #not asked for to the user. this is always 'yes' for now
-		fs_mountpoint=` cut -d ';' -f 3 <<< $fs_string`
-		fs_mount=`      cut -d ';' -f 4 <<< $fs_string` #we dont need to ask this to the user. this is always 'target' for 99.99% of the users
-		fs_opts=`       cut -d ';' -f 5 <<< $fs_string`
-		fs_label=`      cut -d ';' -f 6 <<< $fs_string`
-		fs_params=`     cut -d ';' -f 7 <<< $fs_string`
-		[ "$fs_type"   = no_type             ] && fs_type=
-		[ "$fs_mountpoint"  = no_mountpoint  ] && fs_mountpoint=
-		[ "$fs_mount"  = no_mount            ] && fs_mount=
-		[ "$fs_opts"   = no_opts             ] && fs_opts=
-		[ "$fs_label"  = no_label            ] && fs_label=
-		[ "$fs_params" = no_params           ] && fs_params=
+		parse_filesystem_string "$fs_string" yes
 		local old_fs_type=$fs_type
 		local old_fs_create=$fs_create
 		local old_fs_mountpoint=$fs_mountpoint
 		local old_fs_mount=$fs_mount
-		local old_fs_opts=$fs_opts
+		local old_fs_opts="$fs_opts"
 		local old_fs_label=$fs_label
-		local old_fs_params=$fs_params
+		local old_fs_params="$fs_params"
 
 		ask_option edit "Change $fs_type filesystem settings on $part ?" \
 		                "Change $fs_type filesystem settings (create:$fs_create, label:$fs_label, mountpoint:$fs_mountpoint) on $part (type:$part_type, label:$part_label) ?" required \
@@ -494,9 +477,13 @@ interactive_filesystem ()
 		then
 			# add $part to $fs_params if it's not in there because the user wants this enabled by default. TODO: we should find something out so you can't disable $part. (would be weird to have a vg listed on $part and not have $part it fs_params)
 			pv=${part/+/}
-			grep -q ":$pv:" <<< $fs_params || grep -q ":$pv\$" <<< $fs_params || fs_params="$fs_params:$pv"
+			if ! egrep -q "$pv(\$| )" <<< "$fs_params"; then
+			   [ -n "$fs_params" ] && fs_params="$fs_params "
+			   fs_params="$fs_params$pv"
+		   fi   
+
 			list=
-			for pv in `sed 's/:/ /' <<< $fs_params`
+			for pv in $fs_params
 			do
 				list="$list $pv ^ ON"
 			done
@@ -511,7 +498,7 @@ interactive_filesystem ()
 				fs_params=${list2[0]}
 			else
 				ask_checklist "Which lvm PV's must this volume group span?" $list || return 1
-				fs_params="$(sed 's/ /:/' <<< "$ANSWER_CHECKLIST")" #replace spaces by colon's, we cannot have spaces anywhere in any string
+				fs_params="$ANSWER_CHECKLIST"
 			fi
 		fi
 		if [ "$fs_create" == yes ] && [ "$fs_type" = lvm-lv ]
@@ -525,9 +512,9 @@ interactive_filesystem ()
 		if [ "$fs_create" == yes ] && [ "$fs_type" = dm_crypt ]
 		then
 			[ -z "$fs_params" ] && default='-c aes-xts-plain -y -s 512'
-			[ -n "$fs_params" ] && default="${fs_params//_/ }"
+			[ -n "$fs_params" ] && default="$fs_params"
 			ask_string "Enter the options for this $fs_type on $part" "$default" || return 1
-			fs_params="${ANSWER_STRING// /_}"
+			fs_params="$ANSWER_STRING"
 		fi
 
 		# ask opts
@@ -537,7 +524,7 @@ interactive_filesystem ()
 			[ -n "$fs_opts" ] && default="$fs_opts"
 			program=`get_filesystem_program $fs_type`
 			ask_string "Enter any additional opts for $program" "$default" 0
-			fs_opts=$(sed 's/ /_/g' <<< "$ANSWER_STRING") #TODO: clean up all whitespace (tabs and shit)
+			fs_opts="$ANSWER_STRING"
 		fi
 
 		[ -z "$fs_type"       ] && fs_type=no_type
@@ -545,7 +532,7 @@ interactive_filesystem ()
 		[ -z "$fs_opts"       ] && fs_opts=no_opts
 		[ -z "$fs_label"      ] && fs_label=no_label
 		[ -z "$fs_params"     ] && fs_params=no_params
-		NEW_FILESYSTEM="$fs_type;$fs_create;$fs_mountpoint;target;$fs_opts;$fs_label;$fs_params"
+		NEW_FILESYSTEM="$fs_type;$fs_create;$fs_mountpoint;target;${fs_opts// /__};$fs_label;${fs_params// /__}"
 
 		# add new theoretical blockdevice, if relevant
 		new_device=
@@ -605,8 +592,14 @@ interactive_filesystems() {
 			renew_blockdevices=0
 		fi
 	fi
-	[ "$renew_blockdevices" = 1 ] && findpartitions 0 'no_fs' ' raw no_label' > $TMP_BLOCKDEVICES
+	[ "$renew_blockdevices" = 1 ] && findblockdevices 0 'no_fs' ' raw no_label' > $TMP_BLOCKDEVICES
 
+	[ -z "$PART_ACCESS" ] && PART_ACCESS=dev
+	ask_option $PART_ACCESS 'Partition Access Method' 'How do you want your partitions to be accessed in grubs menu.lst and /etc/fstab?' '' \
+		"dev"	"directly by /dev/* (most intuitive but devicefile names can change on kernel updates)" \
+		"label"	"by Disk-Label (Will use the filesystem labels where you give them, and fall back on 'dev' otherwise)" \
+		"uuid"	"by Universally Unique Identifier (You don't need to do anything, but doesn't look pretty)" || return 1
+	PART_ACCESS=$ANSWER_OPTION
 	ALLOK=0
 	while [ "$ALLOK" = 0 ]
 	do
@@ -811,7 +804,7 @@ interactive_runtime_network() {
 
     if ask_yesno "Do you want to use DHCP?"
     then
-        infofy "Please wait.  Polling for DHCP server on $INTERFACE..."
+        inform "Please wait.  Polling for DHCP server on $INTERFACE..."
         dhcpcd -k $INTERFACE >$LOG 2>&1
         if ! dhcpcd $INTERFACE >$LOG 2>&1
         then
@@ -929,7 +922,7 @@ interactive_grub() {
         interactive_grub_menulst
 
         DEVS=$(finddisks 1 _)
-        DEVS="$DEVS $(findpartitions 1 _)"
+        DEVS="$DEVS $(findblockdevices 1 _)"
         if [ "$DEVS" = " " ]; then
             notify "No hard drives were found"
             return 1
